
#include <stddef.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "btn.h"

volatile uint8_t key_state;				// debounced and inverted key state:
                    					// bit = 1: key pressed
volatile uint8_t key_press;				// key press detect
volatile uint8_t key_release;				// key release detect
volatile uint8_t btn_countdown;
volatile uint8_t btn_int1_countup;
volatile uint32_t btn_seconds;

void btn_init(void)
{
  BTN_DDR  &= ~BTN_KEY_ANY;				// set Input
  BTN_PORT |= BTN_KEY_ANY;                      	// activate pull-up

  TCCR0A    = 2;                                   	// reset TCNT0 at compare match
  TIMSK0   |= 1<<OCIE0A;                    		// activate OCRA Interrupt
  OCR0A     = 250;					// for 4ms

  btn_timer_start();
}

ISR (TIMER0_COMPA_vect)			// every 4ms
{
  static uint8_t ct0, ct1, timer_countdown;
  uint8_t j;

  j = key_state ^ ~BTN_PIN;		// key changed ?
  ct0 = ~( ct0 & j );			// reset or count ct0
  ct1 = ct0 ^ (ct1 & j);		// reset or count ct1
  j &= ct0 & ct1;			    // count until roll over ?
  key_state ^= j;		     	// then toggle debounced state
  key_press    |= key_state & j;	// 0->1: key press detect
  key_release |= ~key_state & j;  	// 1->0: key release detect  
 
  if(btn_countdown) btn_countdown--;

  if(btn_int1_countup != 0xFF) btn_int1_countup;

  if(--timer_countdown == 0)
     {
       timer_countdown = 250;
       btn_seconds++;
     }
}
